package multicusecase.descriptor;

import org.ercim.gridcomp.am.controller.AutonomicController;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.core.ProActiveRuntimeException;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.ContentDescription;
import org.objectweb.proactive.core.component.ControllerDescription;
import org.objectweb.proactive.core.component.Fractive;
import org.objectweb.proactive.core.component.ProActiveInterface;
import org.objectweb.proactive.core.component.controller.AbstractProActiveController;
import org.objectweb.proactive.core.component.controller.ComponentParametersController;
import org.objectweb.proactive.core.component.controller.MulticastController;
import org.objectweb.proactive.core.component.group.ProxyForComponentInterfaceGroup;
import org.objectweb.proactive.core.component.representative.ProActiveComponentRepresentative;
import org.objectweb.proactive.core.component.type.ProActiveTypeFactoryImpl;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

public class AutonomicControllerMulticastItfsImpl extends AbstractProActiveController
									 implements AutonomicController{
	
	private String className = AutonomicControllerMulticastItfsImpl.class.getSimpleName()+"> ";
	private void out(String s){	System.out.println(className+s); }
	
	public AutonomicControllerMulticastItfsImpl(Component owner) {
		super(owner);
	}
	
	private static final long serialVersionUID = 1L;

	@Override
	protected void setControllerItfType() {
		try {
			setItfType(ProActiveTypeFactoryImpl.instance().createFcItfType(
					AutonomicController.CONTROLLER_NAME,
					AutonomicController.class.getName(),
					TypeFactory.SERVER,
					TypeFactory.OPTIONAL,
					TypeFactory.SINGLE));
		} catch (InstantiationException e) {
			throw new ProActiveRuntimeException("Cannot create controller type: " + getClass().getName());
		}
	}

	public GenericTypeWrapper execOperation(String op, Object... params) {
		if(op.equals("increaseParallelDegree")){
			try {
				Fractal.getLifeCycleController(owner).stopFc();
			} catch (IllegalLifeCycleException e1) {
				e1.printStackTrace();
			} catch (NoSuchInterfaceException e1) {
				e1.printStackTrace();
			}
			
			String workerItfName = null;
			String multicastItf = null;
			
			// is a member of a collection itf?
			InterfaceType[] itfTypes = ((ComponentType)owner.getFcType()).getFcInterfaceTypes();
			for (int i = 0; i < itfTypes.length; i++) {
				InterfaceType type = itfTypes[i];
				multicastItf = type.getFcItfName();
				
				try {
					MulticastController ownerBC = Fractive.getMulticastController(owner);
					ProxyForComponentInterfaceGroup pfc = (ProxyForComponentInterfaceGroup)ownerBC.lookupFcMulticast(type.getFcItfName());
					if (pfc==null) 
						continue;
					ProActiveInterface pai = (ProActiveInterface) (pfc.getDelegatee().getMemberList().elementAt(0));
					workerItfName = pai.getFcItfName();
				} catch (NoSuchInterfaceException e) {
					e.printStackTrace();
				}
				
			}
			
			
			ContentController ownerCC;
			try {
				ownerCC = (ContentController) owner.getFcInterface("content-controller");
				Component internal = ownerCC.getFcSubComponents()[0];
				
				Component newC = newComponent(internal);
				ownerCC.addFcSubComponent(newC);
				((LifeCycleController)newC.getFcInterface("lifecycle-controller")).stopFc();
				
				MulticastController ownerMC = (MulticastController) owner.getFcInterface("multicast-controller");
				ownerMC.bindFcMulticast(multicastItf, 
										(ProActiveInterface)newC.getFcInterface(workerItfName));
				
			} catch (NoSuchInterfaceException e) {
				e.printStackTrace();
			} catch (IllegalContentException e) {
				e.printStackTrace();
			} catch (IllegalLifeCycleException e) {
				e.printStackTrace();
			} 
		}
		
		try {
			Fractal.getLifeCycleController(owner).startFc();
		} catch (IllegalLifeCycleException e1) {
			e1.printStackTrace();
		} catch (NoSuchInterfaceException e1) {
			e1.printStackTrace();
		}
		return null;
	}
	
	private Component newComponent(Component toParallelize) {
		// Retreiving Type & Component Factories
		Component boot = null;
		try {
			boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}

		GenericFactory cf = null;
		try {
			cf = Fractal.getGenericFactory(boot);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Cloning controllers
		ControllerDescription controlDesc = null;
		try {
			controlDesc = ((ComponentParametersController) toParallelize.getFcInterface(Constants.COMPONENT_PARAMETERS_CONTROLLER)).
			getComponentParameters().
			getControllerDescription();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Creating a new Name
		NameController toParallelizeNC = null;
		try {
			toParallelizeNC = (NameController) toParallelize.getFcInterface("name-controller");
		} catch (NoSuchInterfaceException e2) {
			e2.printStackTrace();
		}
		String name  = toParallelizeNC.getFcName()+"-autonomic-duplicate-0";
		controlDesc.setName(name);

		// Cloning content
		String clazz = ((ProActiveComponentRepresentative)toParallelize).getStubOnBaseObject().getClass().getSuperclass().getName();
		ContentDescription contentDesc = new ContentDescription(clazz);
		
		Component toParallelizeClone = null;
		try {
			toParallelizeClone = cf.newFcInstance( toParallelize.getFcType(), 
					controlDesc,
					contentDesc );
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
		
		LifeCycleController tPcLCC;
		try {
			tPcLCC = (LifeCycleController) toParallelizeClone.getFcInterface("lifecycle-controller");
			tPcLCC.stopFc();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			e.printStackTrace();
		}
		
		return toParallelizeClone;
	}
	

	public Double getBandwidth() {
		if(owner.getReferenceOnBaseObject() instanceof AutonomicController){
			out("Oggetto: "+owner.getReferenceOnBaseObject().getClass().getSuperclass().getCanonicalName()+" compatibile con l'interfaccia AutonomicController");
			return ((AutonomicController) owner.getReferenceOnBaseObject()).getBandwidth();
		}
		
		return 0.0;
	}

	public Integer getParallelismDegree() {
		if(owner.getReferenceOnBaseObject() instanceof AutonomicController){
			out("Object: "+owner.getReferenceOnBaseObject().getClass().getSuperclass().getCanonicalName()+" compatibile con l'interfaccia AutonomicController");
			return ((AutonomicController) owner.getReferenceOnBaseObject()).getParallelismDegree();
		}
		return 0;
	}

	public String[] listAutonomicOperations() {
		
		return new String[]{"increaseParallelDegree"};
	}

	public void setBandwidth(double bandwidth) {
		if(owner.getReferenceOnBaseObject() instanceof AutonomicController){
			out("Object: "+owner.getReferenceOnBaseObject().getClass().getSuperclass().getCanonicalName()+" compatibile con l'interfaccia AutonomicController");
			((AutonomicController) owner.getReferenceOnBaseObject()).setBandwidth(bandwidth);
		}

	}

	public void setParallelismDegree(int parDegree) {
		if(owner.getReferenceOnBaseObject() instanceof AutonomicController){
			out("Object: "+owner.getReferenceOnBaseObject().getClass().getSuperclass().getCanonicalName()+" compatibile con l'interfaccia AutonomicController");
			((AutonomicController) owner.getReferenceOnBaseObject()).setParallelismDegree(parDegree);
		}
	}

}
