/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2002 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive-support@inria.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://www.inria.fr/oasis/ProActive/contacts.html
 *  Contributor(s):
 *
 * ################################################################
 */
package nonregressiontest.component.binding.local.collective;

import java.util.Arrays;

import nonregressiontest.component.ComponentTest;
import nonregressiontest.component.I1;
import nonregressiontest.component.I2;
import nonregressiontest.component.Message;
import nonregressiontest.component.PrimitiveComponentB;
import nonregressiontest.component.PrimitiveComponentD;
import nonregressiontest.component.PrimitiveComponentE;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.ProActive;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.ContentDescription;
import org.objectweb.proactive.core.component.ControllerDescription;
import org.objectweb.proactive.core.component.type.Composite;
import org.objectweb.proactive.core.group.ProActiveGroup;

import testsuite.test.Assertions;


/**
 * @author Matthieu Morel
 *
 * a test for bindings on client collective interfaces between remote components
 */
public class Test extends ComponentTest {
    public static String MESSAGE = "-->Main";
    Component pD1;
    Component pB1;
    Component pB2;
    Message message;

    public Test() {
        super("Communication between local primitive components through client collection interface",
            "Communication between local primitive components through client collection interface ");
    }

    /**
     * @see testsuite.test.FunctionalTest#action()
     */
    public void action() throws Exception {
        Component boot = Fractal.getBootstrapComponent();
        TypeFactory type_factory = Fractal.getTypeFactory(boot);
        GenericFactory cf = Fractal.getGenericFactory(boot);
        ComponentType D_Type = type_factory.createFcType(new InterfaceType[] {
                    type_factory.createFcItfType("i1", I1.class.getName(),
                        TypeFactory.SERVER, TypeFactory.MANDATORY,
                        TypeFactory.SINGLE),
                    type_factory.createFcItfType("i2", I2.class.getName(),
                        TypeFactory.CLIENT, TypeFactory.MANDATORY,
                        TypeFactory.COLLECTION)
                });
        ComponentType B_Type = type_factory.createFcType(new InterfaceType[] {
                    type_factory.createFcItfType("i2", I2.class.getName(),
                        TypeFactory.SERVER, TypeFactory.MANDATORY,
                        TypeFactory.SINGLE)
                });
        
        ComponentType eType = type_factory.createFcType(new InterfaceType[] {
                type_factory.createFcItfType("i1", I1.class.getName(),
                    TypeFactory.SERVER, TypeFactory.MANDATORY,
                    TypeFactory.SINGLE),
                type_factory.createFcItfType("i2", I2.class.getName(),
                            TypeFactory.CLIENT, TypeFactory.MANDATORY,
                            TypeFactory.COLLECTION)});
    
        
        ComponentType fType = type_factory.createFcType(new InterfaceType[] {
                    type_factory.createFcItfType("i2-server", I2.class.getName(),
                        TypeFactory.SERVER, TypeFactory.MANDATORY,
                        TypeFactory.COLLECTION),
        type_factory.createFcItfType("i2-client", I2.class.getName(),
                                TypeFactory.CLIENT, TypeFactory.MANDATORY,
                                TypeFactory.COLLECTION)});
        
        

        // instantiate the components
        pD1 = cf.newFcInstance(D_Type,
                new ControllerDescription("pD1", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentD.class.getName(),
                    new Object[] {  }));
        pB1 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB1", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        pB2 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB2", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        
        

        // check that listFc() does not return the name of the collective interface :
        // it should return no client interface
        Assertions.assertTrue(Fractal.getBindingController(pD1).listFc().length == 0);

        // bind the components
        Fractal.getBindingController(pD1).bindFc("i2_01", pB1.getFcInterface("i2"));
        Fractal.getBindingController(pD1).bindFc("i2_02", pB2.getFcInterface("i2"));

        // check that listFc() does not return the name of the collective interface
        Assertions.assertTrue(Arrays.equals(Fractal.getBindingController(pD1).listFc(), new String[] {"i2_01", "i2_02"}));

        // start them
        Fractal.getLifeCycleController(pD1).startFc();
        Fractal.getLifeCycleController(pB1).startFc();
        Fractal.getLifeCycleController(pB2).startFc();

        message = null;
        I1 i1 = (I1) pD1.getFcInterface("i1");
        Message msg1 = i1.processInputMessage(new Message(MESSAGE));
        message = msg1.append(MESSAGE);
        
        
//      test collection itf with composite component
        Component c1 = cf.newFcInstance(fType,
                new ControllerDescription("composite1", Constants.COMPOSITE),
                new ContentDescription(Composite.class.getName(),
                    new Object[] {  }));
        Component pB3 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB3", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        Component pB4 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB4", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        Component pB5 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB5", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        Component pB6 = cf.newFcInstance(B_Type,
                new ControllerDescription("pB6", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentB.class.getName(),
                    new Object[] {  }));
        Component pE = cf.newFcInstance(eType,
                new ControllerDescription("pE", Constants.PRIMITIVE),
                new ContentDescription(PrimitiveComponentE.class.getName(),
                    new Object[] {  }));
        Fractal.getContentController(c1).addFcSubComponent(pB3);
        Fractal.getContentController(c1).addFcSubComponent(pB4);
        Fractal.getContentController(c1).addFcSubComponent(pE);
        Fractal.getBindingController(c1).bindFc("i2-server-01", pB3.getFcInterface("i2"));
        Fractal.getBindingController(c1).bindFc("i2-server-02", pB4.getFcInterface("i2"));
        Fractal.getBindingController(pE).bindFc("i2-01", c1.getFcInterface("i2-client-01"));
        Fractal.getBindingController(pE).bindFc("i2-02", c1.getFcInterface("i2-client-02"));
        Fractal.getBindingController(c1).bindFc("i2-client-01", pB5.getFcInterface("i2"));
        Fractal.getBindingController(c1).bindFc("i2-client-02", pB6.getFcInterface("i2"));
     
        Fractal.getLifeCycleController(c1).startFc();
        Fractal.getLifeCycleController(pB5).startFc();
        Fractal.getLifeCycleController(pB6).startFc();
        ((I1)pE.getFcInterface("i1")).processInputMessage(new Message(""));
        Message expected = new Message("composite-"+PrimitiveComponentB.MESSAGE);
        Message m1 = ((I2)c1.getFcInterface("i2-server-01")).processOutputMessage(new Message("composite-"));
        Message m2 = ((I2)c1.getFcInterface("i2-server-02")).processOutputMessage(new Message("composite-"));
        Assertions.assertEquals(new Message("composite-"+PrimitiveComponentB.MESSAGE).toString(), ProActive.getFutureValue(m1).toString());
        Assertions.assertEquals(new Message("composite-"+PrimitiveComponentB.MESSAGE).toString(), ProActive.getFutureValue(m2).toString());
    }

    /**
     * @see testsuite.test.AbstractTest#initTest()
     */
    public void initTest() throws Exception {
    }

    /**
     * @see testsuite.test.AbstractTest#endTest()
     */
    public void endTest() throws Exception {
    }

    public boolean postConditions() throws Exception {
        StringBuffer resulting_msg = new StringBuffer();
        int message_size = ProActiveGroup.size(message);
        for (int i = 0; i < message_size; i++) {
            resulting_msg.append(((Message) ProActiveGroup.get(message, i)).toString());
        }
        // this --> primitiveA --> primitiveB --> primitiveA --> this  (message goes through composite components)
        String single_message = Test.MESSAGE + PrimitiveComponentD.MESSAGE +
            PrimitiveComponentB.MESSAGE + PrimitiveComponentD.MESSAGE +
            Test.MESSAGE;
        return resulting_msg.toString().equals(single_message + single_message);
    }
    
    
    
    
    public static void main(String[] args) {
        Test test = new Test();
        try {
            test.action();
            if (test.postConditions()) {
                System.out.println("TEST SUCCEEDED");
            } else {
                System.out.println("TEST FAILED");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
