package org.ercim.gridcomp.am;


import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Vector;

import javax.naming.NamingException;

import org.ercim.gridcomp.am.controller.AutonomicController;
import org.ercim.gridcomp.am.controller.AutonomicControllerImpl;
import org.ercim.gridcomp.am.controller.CompositeMembraneAutonomicController;
import org.ercim.gridcomp.am.controller.CollectiveInterfaceDispatcherProxy;
import org.ercim.gridcomp.am.example.Stage;
import org.ercim.gridcomp.am.util.RegistrationSimplifier;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.control.SuperController;

import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;

import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;

import org.objectweb.fractal.util.Fractal;

import org.objectweb.proactive.Body;
import org.objectweb.proactive.RunActive;

import org.objectweb.proactive.core.body.future.Future;
import org.objectweb.proactive.core.component.ComponentParameters;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.ContentDescription;
import org.objectweb.proactive.core.component.ControllerDescription;
import org.objectweb.proactive.core.component.Fractive;
import org.objectweb.proactive.core.component.ProActiveInterface;
import org.objectweb.proactive.core.component.adl.Launcher;
import org.objectweb.proactive.core.component.config.ComponentConfigurationHandler.ControllerHandler;
import org.objectweb.proactive.core.component.controller.ComponentParametersController;
import org.objectweb.proactive.core.component.controller.MulticastController;
import org.objectweb.proactive.core.component.controller.ProActiveContentController;
import org.objectweb.proactive.core.component.controller.ProActiveLifeCycleController;
import org.objectweb.proactive.core.component.factory.ProActiveGenericFactory;
import org.objectweb.proactive.core.component.group.ProxyForComponentInterfaceGroup;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.representative.ProActiveComponentRepresentative;
import org.objectweb.proactive.core.component.type.ProActiveInterfaceType;
import org.objectweb.proactive.core.component.type.ProActiveTypeFactory;
import org.objectweb.proactive.core.mop.ConstructionOfReifiedObjectFailedException;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

import sun.awt.ComponentFactory;

public class AutonomicManagerImpl implements RunActive, AutonomicManager, BindingController{

	private AutonomicController[] bindings = new AutonomicController[3];
	private String[] bindingsNames = new String[3];

	private ProActiveComponent superComponent = null; 
	private ProActiveComponentRepresentative rsComp = null;

	private String className = AutonomicManagerImpl.class.getSimpleName()+"> ";
	private void out(String s){	System.out.println(className+s); }
	private void err(String s){	System.err.println(className+s); }

	private boolean topology = false;
	private boolean binded = false;
	private HashMap<Component, String> comp2Name = null;
	private HashMap<String, Component> name2Comp = null;
	private HashMap<String, Component_IN_OUT_Link> compLinks = null;

	// BINDING CONTROLLER

	private void initBindings() {
		for(int i=0;i<bindingsNames.length;i++){
			bindingsNames[i] = new String("client-autonomic-controller-"+i);
		}
		itfsNameInitialized = true;
	}

	public void bindFc(String arg0, Object arg1) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {

		if(!itfsNameInitialized) initBindings();

		if(arg0.startsWith("client-autonomic-controller-")){
			String num = arg0.substring(arg0.lastIndexOf('-')+1);
			try{
				int itf = Integer.parseInt(num);
				out("ITF ::= "+itf);
				bindings[itf] = (AutonomicController)arg1;
				return;
			}catch(NumberFormatException nfe){
				throw new NoSuchInterfaceException("Parsing itf number. Itf number "+num+" not present.");
			}
		}

	}

	private boolean itfsNameInitialized = false;
	public String[] listFc() {

		if(!itfsNameInitialized) initBindings();

		return bindingsNames;
	}

	public Object lookupFc(String clientItfName) throws NoSuchInterfaceException {

		if(!itfsNameInitialized) initBindings();

		if(clientItfName.startsWith("client-autonomic-controller-")){
			String num = clientItfName.substring(clientItfName.lastIndexOf('-')+1);
			try{
				int itf = Integer.parseInt(num);
				out("ITF ::= "+itf);
				return bindings[itf];
			}catch(NumberFormatException nfe){
				throw new NoSuchInterfaceException("Parsing itf number. Itf number "+num+" not present.");
			}
		}

		throw new NoSuchInterfaceException(clientItfName);
	}

	public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {

		if(!itfsNameInitialized) initBindings();

		if(clientItfName.startsWith("client-autonomic-controller-")){
			String num = clientItfName.substring(clientItfName.lastIndexOf('-')+1);
			try{
				int itf = Integer.parseInt(num);
				out("ITF ::= "+itf);
				bindings[itf] = null;
				return;
			}catch(NumberFormatException nfe){
				throw new NoSuchInterfaceException("Parsing itf number. Itf number "+num+" not present.");
			}
		}

		throw new NoSuchInterfaceException(clientItfName);
	}


	// AUTONOMIC MANAGER

	private void initAutonomicManager(){

		boolean rsFound = false;

		// Registering itself
		try {
			Fractive.register(Fractive.getComponentRepresentativeOnThis(), "AutonomicManager");
		} catch (IOException e2) {
			e2.printStackTrace();
			System.exit(1);
		}

		// Looking for the RegistrationSimplifier component
		while(!rsFound){

			try {
				rsComp = Fractive.lookup("RegistrationSimplifier");
				out("RegistrationSimplifier... found");
				rsFound = true;

			} catch (IOException e1) {
				out("RegistrationSimplifier... still not found");
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			} catch (NamingException e1) {
				e1.printStackTrace();
				System.exit(1);
			}
		}

		// Retreiving the SuperComponent
		Component[] scmps;
		try {
			scmps = ((SuperController) (rsComp.getFcInterface("super-controller"))).getFcSuperComponents();
			superComponent = (ProActiveComponent)scmps[0];
			out("Super-component... found: "+superComponent.toString());

		} catch (NoSuchInterfaceException e1) {
			e1.printStackTrace();
			System.exit(1);
		}
	}

	private boolean initialized = false;
	private ProActiveContentController scCC = null;

	public void runActivity(Body body) {

		// Initializing Autonomic Manager
		if(!initialized) 
			initAutonomicManager();


		// Sleep 1 seconds 
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		try {

			// Discovering topology changes
			Component[] cm = monitorTopology();

			// Printning Components 
			// printComponents(cm);

			// Enforcing NF Requirements
			enforceNFContract(cm);

		} catch (NoSuchInterfaceException e) {

			e.printStackTrace();
		}

		

		catch (Exception e) {
			e.printStackTrace();
		}

	}


	private Component[] monitorTopology() {

		// Capire se ci sono state modifiche alla topologia
		try {
			scCC = Fractive.getProActiveContentController(superComponent);
		} catch (NoSuchInterfaceException e1) {
			e1.printStackTrace();
		}

		// Riumovo dall'array di componenti l'AM e il RegistrationSimplifier
		Component[] subComps = filterComponents(scCC.getFcSubComponents());


		// Se ci sono state modifiche deve aggiornare le informazioni sulla topologia
		if(!topology){

			comp2Name = new HashMap<Component, String>();
			name2Comp = new HashMap<String, Component>();
			compLinks = new HashMap<String, Component_IN_OUT_Link>();

			for(Component cmp : subComps){

				String cmpName = new Integer(cmp.hashCode()).toString();

				comp2Name.put(cmp, cmpName);
				name2Comp.put(cmpName, cmp);
				System.out.println("Il componente aggiunto e' : "+cmpName+" - "+new Integer(cmp.hashCode()).toString());
				compLinks.put(cmpName, new Component_IN_OUT_Link());
			}
			// POST: ogni componente ha un nome associato 


			for(Component cmp : subComps){

				try {
					BindingController cmpBC = (BindingController) cmp.getFcInterface("binding-controller");

					String[] cmpItfs = cmpBC.listFc();
					for(String cmpItf : cmpItfs){

						Interface i = (Interface) cmp.getFcInterface(cmpItf);

						Component_IN_OUT_Link inOut = compLinks.get(comp2Name.get(cmp));
						inOut.OUT.add(new Link( i, ((ProActiveInterface)cmpBC.lookupFc(((ProActiveInterface)i).
								getFcItfName())),
								comp2Name.get( ((ProActiveInterface)cmpBC.
										lookupFc(((ProActiveInterface)i).
												getFcItfName())).
												getFcItfOwner() 
								)
						)
						);
						err(">>>"+((ProActiveInterface)cmpBC.lookupFc(((ProActiveInterface)i).
								getFcItfName())).getFcItfName()
						);

						inOut = compLinks.get(comp2Name.get( ((ProActiveInterface)cmpBC.lookupFc(((ProActiveInterface)i).getFcItfName())).getFcItfOwner() ));

						inOut.IN.add(new Link(i, ((ProActiveInterface)cmpBC.lookupFc(((ProActiveInterface)i).
								getFcItfName())), comp2Name.get(cmp)));

					}

				} catch (NoSuchInterfaceException e) {
					out("Component: "+comp2Name.get(cmp)+" does not have a binding controller interface");
					//e.printStackTrace();
				}

			}

			topology = true;


			for(Map.Entry<String, Component_IN_OUT_Link> ciol : compLinks.entrySet()){
				out(""+ciol.getKey());
				out("\tIN:");
				for(Link l : ciol.getValue().IN){
					out("\t\t"+l.name+" "+l.clientItf.getFcItfName()+" -> "+l.serverItf.getFcItfName());
				}

				out("\tOUT:");
				for(Link l : ciol.getValue().OUT){
					out("\t\t"+l.name+" "+l.clientItf.getFcItfName()+" -> "+l.serverItf.getFcItfName());
				}
			}
		}

		// Deve bindare l'autonomic manager ai componenti che lo permettono
		if(!binded){
			try {
				bindTo(subComps);
				binded = true;
			} catch (IllegalBindingException e) {

				e.printStackTrace();
			} catch (IllegalLifeCycleException e) {

				e.printStackTrace();
			}
		}

		// Deve restituire la topologia interna
		return subComps;
	}

	@SuppressWarnings("unused")
	private void printComponents(Component[] cm) throws NoSuchInterfaceException {
		for(Component cmp : cm){

			Object[] itfs = cmp.getFcInterfaces();

			ProActiveComponent pac = rsComp;

			if(!((ProActiveComponent)cmp).equals(Fractive.getComponentRepresentativeOnThis())){
				if(!((ProActiveComponent)cmp).equals(pac))
					out(((NameController)cmp.getFcInterface(Constants.NAME_CONTROLLER)).getFcName());
			}else{
				out("Autonomic Manager: ");
			}

			for(Interface i : (Interface[])itfs){
				out("\t"+i.getFcItfName());
			}

			try{
				AutonomicController ac = ((AutonomicController)cmp.getFcInterface("autonomic-controller"));
				err(ac.execOperation("getClass", (Object[])null).getObject().toString()+"");
			}catch(NoSuchInterfaceException nsie){
				err("manca l'interfaccia autonomic-controller");
			}
		}

	}

	private void enforceNFContract(Component[] cm) throws Exception {

		// Recupero il componente da sostituire:
		Component c = selectComponent(cm);
		err("Componente da sostituire: "+c);

		if(c != null){

			if(managedParallelDegree(c)){

				
				RegistrationSimplifier rsItf = 
					(RegistrationSimplifier)rsComp.getFcInterface("server-registration-simplifier");

				// setto che ci sto lavorando io; -> il lifecycle dell'autonomic manager non e piu soggetto al controllo del supercomponente
				rsItf.setStartOrStopAM(true);

				System.err.println("prima di recuperare il lifecycle controller");

				// STOPPING SuperComponent

				// Retrieving Supercomponent lifecycle controller
				ProActiveLifeCycleController superCompLCC = 
					(ProActiveLifeCycleController)superComponent.getFcInterface("lifecycle-controller");
				
				System.err.println("recuperato il lifecycle controller");

				// Stopping supercomponent lifecycle
				superCompLCC.stopFc();
				
				/*
				ContentController mngdCC = (ContentController) c.getFcInterface("content-controller");
				Component internal = mngdCC.getFcSubComponents()[0];
				
				Component newC = newComponent(internal);
				
				mngdCC.addFcSubComponent(newC);
				*/
				err("ESEGUITA OPERAZIONE DI MANAGED PARALLEL DEGREE");
				
				AutonomicController cAC = (AutonomicController) c.getFcInterface("autonomic-controller");
				String[] ops =  cAC.listAutonomicOperations();
				
				GenericTypeWrapper gtw = cAC.execOperation("increaseParallelDegree", new String[0]);
				
				// FAccio in modo da chiedere il risultato cosi da evitare l'esecuzione asincrona
				Object result = gtw.getObject();
				System.out.println(result);
				
				for(String op : ops){
					System.out.println("--------------------->"+op);
				}
				
//				 Riavvio il supercomponente
				superCompLCC.startFc();
				System.err.println("terminato startFc");


				// UNSETTING StartOrStopAM

				// Dico che su registration simplifier non ci sono piu io a lavorare --> TO DO: la componente potrebbe star tentando di terminare
				// tutti i componenti, compreso AM mentre ancora il StartOrStop e settato a true. Pur avendo AM terminato il suo compito, non verrebbe
				// terminato
				rsItf.setStartOrStopAM(false);
				System.err.println("terminato setstartorstop");
				
				// stop(c);
				// incrParDegree(c);
				// start(c);

			}else{

				// SETTING StartOrStopAM

				// recupero l'interfaccia di RegistrationSimplifier
				RegistrationSimplifier rsItf = 
					(RegistrationSimplifier)rsComp.getFcInterface("server-registration-simplifier");

				// setto che ci sto lavorando io; -> il lifecycle dell'autonomic manager non e piu soggetto al controllo del supercomponente
				rsItf.setStartOrStopAM(true);

				System.err.println("prima di recuperare il lifecycle controller");

				// STOPPING SuperComponent

				// Retrieving Supercomponent lifecycle controller
				ProActiveLifeCycleController superCompLCC = 
					(ProActiveLifeCycleController)superComponent.getFcInterface("lifecycle-controller");
				
				System.err.println("recuperato il lifecycle controller");

				// Stopping supercomponent lifecycle
				superCompLCC.stopFc();


				// TRANSFORMING Component

				// ritorna un componente parallelo contenente c
				System.err.println("iniziato makecomponentparallel");
				Component cComp = makeComponentParallel(c, scCC);
				System.err.println("terminato makecomponentparallel");

				// RESTARTING SuperComponent

				// Riavvio il supercomponente
				superCompLCC.startFc();
				System.err.println("terminato startFc");


				// UNSETTING StartOrStopAM

				// Dico che su registration simplifier non ci sono piu io a lavorare --> TO DO: la componente potrebbe star tentando di terminare
				// tutti i componenti, compreso AM mentre ancora il StartOrStop e settato a true. Pur avendo AM terminato il suo compito, non verrebbe
				// terminato
				rsItf.setStartOrStopAM(false);
				System.err.println("terminato setstartorstop");
			}

		}

		System.out.println("TERMINATO ENFORCENFCONTRACT");
		System.out.flush();
		Thread.sleep(3000);
	}

	private boolean managedParallelDegree(Component c) {
		
		NameController cNC;
		try {
			cNC = (NameController) c.getFcInterface("name-controller");
			if(cNC.getFcName().startsWith("A-C_")) return true;
		} catch (NoSuchInterfaceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}


	private Component makeComponentParallel(Component toParallelize, ContentController parentCC) throws InstantiationException, NoSuchInterfaceException, 
	IllegalLifeCycleException, IllegalContentException, IllegalBindingException{

		// Retreiving Type & Component Factories
		Component boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
		ProActiveTypeFactory tf = (ProActiveTypeFactory) Fractal.getTypeFactory(boot);
		GenericFactory cf = Fractal.getGenericFactory(boot);
		

		// Copying ***Functional*** Interfaces from the component toParallelize
		LinkedList<InterfaceType> ll_itfs = new LinkedList<InterfaceType>();
		for(int i=0;i<toParallelize.getFcInterfaces().length; i++){

			InterfaceType currentItf = (InterfaceType)((Interface)toParallelize.getFcInterfaces()[i]).getFcItfType();
			if(currentItf.getFcItfName().endsWith("-controller"))
				System.out.println("NF-itf: "+currentItf.getFcItfName());
			else{
				ll_itfs.add(currentItf);
				System.out.println("F-itf: "+currentItf.getFcItfName()+" ... Client ? "+currentItf.isFcClientItf());
			}
			
		}
		InterfaceType[] itfTy = ll_itfs.toArray(new InterfaceType[ll_itfs.size()]);

		// Creating Type from the retrieved interfaces
		ComponentType cCompType = tf.createFcType(itfTy); 

		
		// Creating the PARALLEL component
		Component cComp = cf.newFcInstance(cCompType,
				//new ControllerDescription(cCompName, Constants.COMPOSITE),
				new ControllerDescription("parallelComp", 
										  Constants.COMPOSITE, 
										  "/org/ercim/gridcomp/am/descriptor/CompositeMembraneAutonomicController.xml"),
				new ContentDescription("org.objectweb.proactive.core.component.type.Composite"));

		for(Interface i : (Interface[])cComp.getFcInterfaces()){
			System.err.println("Interfacce di cComp: "+i.getFcItfName());
		}
		
		
		// Stopping the lifecycle of the PARALLEL component
		LifeCycleController cCompLCC = (LifeCycleController) cComp.getFcInterface("lifecycle-controller");
		cCompLCC.stopFc();

		// Reassigning  external bindings
		reassignBindings((ProActiveContentController)parentCC, toParallelize, cComp);

		// Adding toParallelize component to the early created parallel component
		ContentController cc = (ContentController) cComp.getFcInterface("content-controller");
		cc.addFcSubComponent(toParallelize); 
		
		System.out.println("Aggiunto toParallelize al componente composito");
		
		Component c = newComponent(toParallelize);
		
		System.out.println("Generato clone");
		
		cc.addFcSubComponent(c);
		
		System.out.println("Aggiunto c al componente composito");
		
        // Adjusting internal binding to put old component inside of parallel component
		try{
			BindingController cBC = (BindingController) c.getFcInterface("binding-controller");
			BindingController tpBC = (BindingController) toParallelize.getFcInterface("binding-controller");
			for(String itf : cBC.listFc()){
				
				System.out.println(itf);
				tpBC.unbindFc(itf);
				tpBC.bindFc(itf, cComp.getFcInterface(itf));
				cBC.bindFc(itf, cComp.getFcInterface(itf));
			}
		}catch(NoSuchInterfaceException nsie){
			
		}

		
		BindingController cCompBC = (BindingController) cComp.getFcInterface("binding-controller");
		
		for(Interface itf : (Interface[]) cComp.getFcInterfaces()){
			if(!((InterfaceType)itf.getFcItfType()).isFcClientItf() && !itf.getFcItfName().endsWith("-controller")){
				
				ProActiveInterface cCompCollItf = Fractive.createCollectiveClientInterface("collective-"+itf.getFcItfName(),
																						   ((InterfaceType)itf.getFcItfType()).getFcItfSignature(), 
																						   cComp);
				
				
				CollectiveInterfaceDispatcherProxy mp = null;
				try {
					
					mp = new CollectiveInterfaceDispatcherProxy(((InterfaceType)itf.getFcItfType()).getFcItfSignature());
					mp.setInterfaceType((ProActiveInterfaceType)itf.getFcItfType());
					
				} catch (ConstructionOfReifiedObjectFailedException e) {
					e.printStackTrace();
				}
				
				mp.add(toParallelize.getFcInterface(itf.getFcItfName()));
				mp.add(c.getFcInterface(itf.getFcItfName()));
				//mp.add(toParallelize.getFcInterface(itf.getFcItfName()));
				//mp.add(toParallelize.getFcInterface(itf.getFcItfName()));
				
				cCompCollItf.setProxy(mp);
				//cCompBC.unbindFc(itf.getFcItfName());
				cCompBC.bindFc(itf.getFcItfName(), cCompCollItf);
			}
		}
				
		return cComp;
	}

	private int counter = 0;
	private Component newComponent(Component toParallelize) {
		
		// Retreiving Type & Component Factories
		Component boot = null;
		try {
			boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}

		GenericFactory cf = null;
		try {
			cf = Fractal.getGenericFactory(boot);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Cloning controllers
		ControllerDescription controlDesc = null;
		try {
			controlDesc = ((ComponentParametersController) toParallelize.getFcInterface(Constants.COMPONENT_PARAMETERS_CONTROLLER)).
			getComponentParameters().
			getControllerDescription();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Creating a new Name
		NameController toParallelizeNC = null;
		try {
			toParallelizeNC = (NameController) toParallelize.getFcInterface("name-controller");
		} catch (NoSuchInterfaceException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		String name  = toParallelizeNC.getFcName()+"-autonomic-duplicate-0";
		controlDesc.setName(name);

		// Cloning content
		String clazz = ((ProActiveComponentRepresentative)toParallelize).getStubOnBaseObject().getClass().getSuperclass().getName();
		ContentDescription contentDesc = new ContentDescription(clazz);
		
		Component toParallelizeClone = null;
		try {
			toParallelizeClone = cf.newFcInstance( toParallelize.getFcType(), 
					controlDesc,
					contentDesc );
		} catch (InstantiationException e) {
			e.printStackTrace();
		}
		
		LifeCycleController tPcLCC;
		try {
			tPcLCC = (LifeCycleController) toParallelizeClone.getFcInterface("lifecycle-controller");
			tPcLCC.stopFc();
		} catch (NoSuchInterfaceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		return toParallelizeClone;
		
		/*
		try {
			Component boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
			ComponentType type = (ComponentType)toParallelize.getFcType();

			String klass = ((ProActiveComponentRepresentative)toParallelize).getStubOnBaseObject().getClass().getSuperclass().getName();
			ContentDescription contentDesc = new ContentDescription(klass);

			String name  = "autonomic-duplicate-" + (++counter);
			
			ControllerDescription controlDesc = 
				((ComponentParametersController)toParallelize.
						getFcInterface(Constants.COMPONENT_PARAMETERS_CONTROLLER)).
						getComponentParameters().
						getControllerDescription();
			
			controlDesc.setName(name);

			Component comp = ((ProActiveGenericFactory)Fractal.getGenericFactory(boot)).newFcInstance(type, controlDesc, contentDesc);
			
			return comp;
		
		} catch (Exception e) {
			e.printStackTrace();
			
		}
		
		return null;
		*/
	}
	
	
	private Component selectComponent(Component[] cm) {

		for(Component comp : cm){
			if(comp instanceof ProActiveComponent){
				
				NameController compNC;
				try {
					compNC = (NameController) comp.getFcInterface("name-controller");
					if(compNC.getFcName().startsWith("A-C_")) return comp;
				} catch (NoSuchInterfaceException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				
			}else if(comp instanceof ProActiveComponentRepresentative){
				NameController compNC;
				try {
					compNC = (NameController) comp.getFcInterface("name-controller");
					if(compNC.getFcName().startsWith("A-C_")) return comp;
				} catch (NoSuchInterfaceException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
		}

		return null;
	}

	private Component[] filterComponents(Component[] fcSubComponents) {

		ProActiveComponent pac=rsComp;
		Vector<Component> vComps = new Vector<Component>();

		for(Component cmp : fcSubComponents){

			if(!((ProActiveComponent)cmp).equals(Fractive.getComponentRepresentativeOnThis())){
				if(!((ProActiveComponent)cmp).equals(pac))
					vComps.add(cmp);
			}
		}

		return vComps.toArray(new Component[vComps.size()]);
	}

	private void reassignBindings(ProActiveContentController pcc, Component oldComp, Component comp) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException, IllegalContentException {

		out("1) Vecchio componente: "+comp2Name.get(oldComp)+"  nuovo componente: "+comp2Name.get(comp));

		Component_IN_OUT_Link ciol = compLinks.get(comp2Name.get(oldComp));

		// Sostituisco il vecchio componente nella tabella dei nomi delle componenti
		String oldCompName = comp2Name.remove(oldComp);
		comp2Name.put(comp, oldCompName);

		out("2) Vecchio componente: "+comp2Name.get(oldComp)+"  nuovo componente: "+comp2Name.get(comp));

		pcc.addFcSubComponent(comp);

		for(Link inList : ciol.IN){
			System.err.println("INLIST: "+inList.name);
			BindingController bc = (BindingController)
			((ProActiveComponent)name2Comp.get(inList.name)).getFcInterface("binding-controller");
			System.out.println("Sostituisco il collegamento in INPUT di nome: "+inList.clientItf.getFcItfName());
			bc.unbindFc(inList.clientItf.getFcItfName());
			bc.bindFc(inList.clientItf.getFcItfName(), comp.getFcInterface(inList.serverItf.getFcItfName()));
		}


		for(Link outList : ciol.OUT){
			System.err.println("OUTLIST: "+outList.name);
			BindingController bc = (BindingController)
			((ProActiveComponent)comp).getFcInterface("binding-controller"); 
			//bc.unbindFc(outList.clientItf.getFcItfName());
			System.out.println("Sostituisco il collegamento in OUTPUT di nome: "+outList.clientItf.getFcItfName());
			bc.bindFc(outList.clientItf.getFcItfName(), ((ProActiveComponent)name2Comp.get(outList.name)).getFcInterface(outList.serverItf.getFcItfName())); //comp.getFcInterface(outList.serverItf.getFcItfName()));
		}


		name2Comp.remove(oldCompName);
		name2Comp.put(oldCompName, comp);

		pcc.removeFcSubComponent(oldComp);

		System.out.println("TERMINATO METODO REASSIGNBINDINGS");
		System.out.flush();

	}

	private void bindTo(Component[] cmps) throws IllegalBindingException, IllegalLifeCycleException{

		int assigned = 0;

		// bindo ogni sotto-componente del super-componente all'autonomic manager
		// purche le sotto-componenti abbiano l'interfaccia NF AutonomicController
		for(int i=0;i<cmps.length;i++){

			// evito di provare a bindarmi a me stesso
			if(!(cmps[i] instanceof AutonomicManager)){
				try {
					AutonomicController autCont = (AutonomicController) cmps[i].getFcInterface("autonomic-controller");
					bindFc("client-autonomic-controller-"+(assigned++), autCont);

				} catch (NoSuchInterfaceException e) {
					err("Il componente numero "+i+" non ha autonomic-controller");
				}
			}
		}

		out("Tutti i binding Completati");
	}

}