package org.ercim.gridcomp.am;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ercim.gridcomp.am.example.Compute1;
import org.ercim.gridcomp.am.example.Compute2;
import org.ercim.gridcomp.am.example.Compute3;
import org.ercim.gridcomp.am.example.Compute4;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.CtMethod;
import javassist.CtNewConstructor;
import javassist.CtNewMethod;
import javassist.Modifier;
import javassist.NotFoundException;

public class DispatcherClassGenerator 
{
	private static ClassPool pool = ClassPool.getDefault();
	private static DispatcherClassGenerator instance = null;
	
	public static final String generatedClassFullName = "org.ercim.gridcomp.am.generated.Dispatcher";
	
	private Class loadClass(final String className) throws ClassNotFoundException 
	{
		return Thread.currentThread().getContextClassLoader().loadClass(className);
	}
	
	private DispatcherClassGenerator() {}
	
	public static DispatcherClassGenerator instance()
	{
		if (instance == null)
			instance = new DispatcherClassGenerator();
		return instance;
	}
	
	public Class generateClass(ComponentType type)
	{
		try {
			Class klass = null;
			try {
				klass = loadClass(generatedClassFullName);
			} catch (ClassNotFoundException cnfe) {
				CtClass generated = pool.makeClass(generatedClassFullName);
				List<CtClass> interfacesToImplement = new ArrayList<CtClass>();
				// Add server interfaces to class declaration
				for (InterfaceType itfType: type.getFcInterfaceTypes()) {
					if (!itfType.isFcClientItf()) {
						CtClass itfClass = pool.get(itfType.getFcItfSignature());
						interfacesToImplement.add(itfClass);
						generated.addInterface(itfClass);
					}
				}
				// Add Serializable interface to class declaration
				generated.addInterface(pool.get(Serializable.class.getName()));
				// Add BindingController interface to class declaration
				generated.addInterface(pool.get(BindingController.class.getName()));
				
				createFields(generated, interfacesToImplement);
	        	generated.addConstructor(CtNewConstructor.defaultConstructor(generated));
				createBindingControllerMethods(generated, interfacesToImplement);
				createComponentMethods(generated, interfacesToImplement);
				
				System.out.println(generated.toString()+"---");
				
				byte[] bytecode = generated.toBytecode();
				klass = defineClass(generatedClassFullName, bytecode);
			}
			return klass;
		} catch (Exception e) {
			throw new RuntimeException("Cannot generate dispatcher with Javassist", e);
		}
	}

	private void createFields(CtClass generated, List<CtClass> interfacesToImplement) throws CannotCompileException, NotFoundException
	{
		CtField field;
    	for (CtClass itf: interfacesToImplement) {
    		field = new CtField(pool.get(Map.class.getName()), getList(itf), generated);
    		field.setModifiers(Modifier.PRIVATE);
			generated.addField(field, "new java.util.Hashtable()");
			field = new CtField(CtClass.intType, getInt(itf), generated);
			generated.addField(field, "-1");
    	}
		
	}

	private void createBindingControllerMethods(CtClass generated, List<CtClass> interfacesToImplement) throws CannotCompileException
	{
		StringBuffer bindFcBuffer, unbindFcBuffer, lookupFcBuffer, listFcBuffer, addBuffer;
		CtMethod method;

		bindFcBuffer   = new StringBuffer("public void bindFc(String clientItfName, Object serverItf) throws org.objectweb.fractal.api.NoSuchInterfaceException, org.objectweb.fractal.api.control.IllegalBindingException, org.objectweb.fractal.api.control.IllegalLifeCycleException {\n");		
    	listFcBuffer   = new StringBuffer("public String[] listFc() {\n    String[] clientItfs = new String[ 0");
    	addBuffer      = new StringBuffer("];\n    java.util.List itfs = new java.util.ArrayList();\n");
    	lookupFcBuffer = new StringBuffer("public Object lookupFc(String clientItfName) throws org.objectweb.fractal.api.NoSuchInterfaceException {\n");
    	unbindFcBuffer = new StringBuffer("public void unbindFc(String clientItfName, Object serverItf) throws org.objectweb.fractal.api.NoSuchInterfaceException, org.objectweb.fractal.api.control.IllegalBindingException, org.objectweb.fractal.api.control.IllegalLifeCycleException {\n");
    	

    	for (CtClass itf: interfacesToImplement) {
    		String name = itf.getName();
    		String map  = getList(itf);

    		bindFcBuffer.append("    if (clientItfName.startsWith(" + name + ".CLIENT_ITF_NAME))\n");
    		bindFcBuffer.append("        " + map + ".put(clientItfName, (" + name + ")serverItf);\n");
    		bindFcBuffer.append("    else \n");

    		listFcBuffer.append(" + " + map + ".size()");

    		addBuffer.append("    itfs.addAll((java.util.Collection) " + map + ");\n");

    		lookupFcBuffer.append("    if (clientItfName.startsWith(" + name + ".CLIENT_ITF_NAME))\n");
    		lookupFcBuffer.append("        if (" + map + ".containsKey(clientItfName))");
    		lookupFcBuffer.append("            return " + map + ".get(clientItfName);\n");


    		unbindFcBuffer.append("    if (clientItfName.startsWith(" + name + ".CLIENT_ITF_NAME)) {\n");
    		unbindFcBuffer.append("        if (" + map + ".containsKey(clientItfName))");
    		unbindFcBuffer.append("            " + map + ".remove(clientItfName);\n");
    		unbindFcBuffer.append("    } else \n");
    	}

    	bindFcBuffer.append("        throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");

    	listFcBuffer.append(addBuffer.append("    return (String[]) itfs.toArray(clientItfs);\n}"));

    	lookupFcBuffer.append("    throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");

    	unbindFcBuffer.append("        throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");

    	method = CtNewMethod.make(  bindFcBuffer.toString(), generated); generated.addMethod(method);
    	method = CtNewMethod.make(  listFcBuffer.toString(), generated); generated.addMethod(method);
    	method = CtNewMethod.make(lookupFcBuffer.toString(), generated); generated.addMethod(method);
    	method = CtNewMethod.make(unbindFcBuffer.toString(), generated); generated.addMethod(method);
	}

	private void createComponentMethods(CtClass generated, List<CtClass> interfacesToImplement) throws CannotCompileException, NotFoundException
	{
		StringBuffer body;
		CtMethod generatedMethod;
		for (CtClass itf: interfacesToImplement) {
			String name = itf.getName();
			String map  = getList(itf);
			String idx  = getInt(itf);
			CtMethod[] declaredMethods = itf.getDeclaredMethods();
            for (CtMethod currentMethod: declaredMethods) {
            	body = new StringBuffer();
            	body.append("{\n" + name + " tmp = (" + name + ")(new java.util.ArrayList(" + map +".values())).get((++" + idx + ") % " + map + ".size());\n");
            	body.append("return tmp." + currentMethod.getName() + "($$);\n}");
            	generatedMethod = CtNewMethod.make(
            			currentMethod.getReturnType(),
            			currentMethod.getName(),
            			currentMethod.getParameterTypes(),
            			currentMethod.getExceptionTypes(),
            			body.toString(),
            			generated);
            	generated.addMethod(generatedMethod);
            }
		}
	}

    public static Class defineClass(final String className, final byte[] bytes) throws ClassNotFoundException, SecurityException, NoSuchMethodException,	IllegalArgumentException, IllegalAccessException, InvocationTargetException 
    {
    	// The following code invokes defineClass on the current thread classloader by reflection
        Class clc = Class.forName("java.lang.ClassLoader");
        Class[] argumentTypes = new Class[4];
        argumentTypes[0] = className.getClass();
        argumentTypes[1] = bytes.getClass();
        argumentTypes[2] = Integer.TYPE;
        argumentTypes[3] = Integer.TYPE;

        Method method = clc.getDeclaredMethod("defineClass", argumentTypes);
        method.setAccessible(true);

        Object[] effectiveArguments = new Object[4];
        effectiveArguments[0] = className;
        effectiveArguments[1] = bytes;
        effectiveArguments[2] = new Integer(0);
        effectiveArguments[3] = new Integer(bytes.length);

        return (Class) method.invoke(Thread.currentThread().getContextClassLoader(), effectiveArguments);
}

	
	/*
	public static CtClass generateCtClass(ComponentType type) throws NotFoundException
	{
        CtClass generated = pool.makeClass(generatedClassName);

        List<CtClass> interfacesToImplement = new ArrayList<CtClass>();

    	CtField field;
    	StringBuffer buf;
    	CtMethod method;

        // Add server interfaces to class declaration
        for (InterfaceType itfType: type.getFcInterfaceTypes()) {
        	if (!itfType.isFcClientItf()) {
            	CtClass itfClass = pool.get(itfType.getFcItfSignature());
            	interfacesToImplement.add(itfClass);
            	generated.addInterface(itfClass);
        	}
        }
        // Add Serializable interface to class declaration
        generated.addInterface(pool.get(Serializable.class.getName()));
        // Add BindingController interface to class declaration
        generated.addInterface(pool.get(BindingController.class.getName()));

        try {
        	// Adding private variables
        	for (CtClass itf: interfacesToImplement) {
        		field = new CtField(pool.get(Map.class.getName()), getList(itf), generated);
        		field.setModifiers(Modifier.PRIVATE);
				generated.addField(field, "new java.util.Hashtable()");
				field = new CtField(CtClass.intType, getInt(itf), generated);
				generated.addField(field, "-1");
        	}
        
        	// Adding empty constructor
        	generated.addConstructor(CtNewConstructor.defaultConstructor(generated));
        	
        	// Adding bindFc method
        	buf = new StringBuffer("public void bindFc(String clientItfName, Object serverItf) throws org.objectweb.fractal.api.NoSuchInterfaceException, org.objectweb.fractal.api.control.IllegalBindingException, org.objectweb.fractal.api.control.IllegalLifeCycleException {\n");
        	for (CtClass itf: interfacesToImplement) {
        		buf.append("    if (clientItfName.startsWith(" + itf.getName() + ".CLIENT_ITF_NAME))\n");
        		buf.append("        " + getList(itf) + ".put(clientItfName, (" + itf.getName() + ")serverItf);\n");
        		buf.append("    else \n");
        	}
        	buf.append("        throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");
        	method = CtNewMethod.make(buf.toString(), generated);
        	generated.addMethod(method);
        	
        	// Adding listFc method
        	buf = new StringBuffer("public String[] listFc() {\n");
        	buf.append("    String[] clientItfs = new String[ 0");
        	for (CtClass itf: interfacesToImplement) 
        		buf.append(" + " + getList(itf) + ".size()");
        	buf.append("];\n");
        	buf.append("    java.util.List itfs = new java.util.ArrayList();\n");
        	for (CtClass itf: interfacesToImplement) 
        		buf.append("    itfs.addAll((java.util.Collection) " + getList(itf) + ");\n");
        	buf.append("    return (String[]) itfs.toArray(clientItfs);\n}");
        	method = CtNewMethod.make(buf.toString(), generated);
        	generated.addMethod(method);
        	
        	// Adding lookupFc method
        	buf = new StringBuffer("public Object lookupFc(String clientItfName) throws org.objectweb.fractal.api.NoSuchInterfaceException {\n");
        	for (CtClass itf: interfacesToImplement) {
        		buf.append("    if (clientItfName.startsWith(" + itf.getName() + ".CLIENT_ITF_NAME))\n");
        		buf.append("        if (" + getList(itf) + ".containsKey(clientItfName))");
        		buf.append("            return " + getList(itf) + ".get(clientItfName);\n");
        	}
        	buf.append("    throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");
        	method = CtNewMethod.make(buf.toString(), generated);
        	generated.addMethod(method);

        	// Adding unbindFc method
        	buf = new StringBuffer("public void unbindFc(String clientItfName, Object serverItf) throws org.objectweb.fractal.api.NoSuchInterfaceException, org.objectweb.fractal.api.control.IllegalBindingException, org.objectweb.fractal.api.control.IllegalLifeCycleException {\n");
        	for (CtClass itf: interfacesToImplement) {
        		buf.append("    if (clientItfName.startsWith(" + itf.getName() + ".CLIENT_ITF_NAME)) {\n");
        		buf.append("        if (" + getList(itf) + ".containsKey(clientItfName))");
        		buf.append("            " + getList(itf) + ".remove(clientItfName);\n");
        		buf.append("    } else \n");
        	}
        	buf.append("        throw new org.objectweb.fractal.api.NoSuchInterfaceException(clientItfName);\n}");
        	method = CtNewMethod.make(buf.toString(), generated);
        	generated.addMethod(method);
        	
        	
            // Adding server methods
    		for (CtClass itf: interfacesToImplement) {
    			CtMethod[] declaredMethods = itf.getDeclaredMethods();
                for (CtMethod currentMethod: declaredMethods) {
                	method = new CtMethod(currentMethod.getReturnType(),
                			currentMethod.getName(),
                			currentMethod.getParameterTypes(),
                			generated);
                	buf = new StringBuffer("{\n");
                	buf.append("    " + itf.getName() + " tmp = (new java.util.ArrayList(" + getList(itf) +".values())).get((++" + getInt(itf) + ") % " + getList(itf) + ".size());\n");
                	buf.append("    return " + itf.getName() + "." + currentMethod.getName() + "($$);");
                	buf.append("}\n");
                	method = CtNewMethod.make(
                			currentMethod.getReturnType(),
                			currentMethod.getName(),
                			currentMethod.getParameterTypes(),
                			currentMethod.getExceptionTypes(),
                			buf.toString(),
                			generated
                	);
                	generated.addMethod(method);
                }
    		}
    		
        	
		} catch (CannotCompileException e) {
			e.printStackTrace();
		}
		return generated;
	}
	
	public static Class generateClass(ComponentType type) throws CannotCompileException, NotFoundException
	{
		return generateCtClass(type).toClass();
	}
	
	*/
	private static String getList(CtClass itf)
	{
		return "my" + itf.getSimpleName() + "Map";
	}

	private static String getInt(CtClass itf)
	{
		return "rr" + itf.getSimpleName() + "Counter";
	}

	public static void main(String args[]) throws CannotCompileException, InstantiationException, NoSuchInterfaceException, NotFoundException 
	{
        Component boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
        TypeFactory tf = Fractal.getTypeFactory(boot);

        ComponentType type = tf.createFcType(new InterfaceType[] {
        	tf.createFcItfType("s1", Compute1.class.getName(), false, false, false),
        	tf.createFcItfType("s2", Compute2.class.getName(), false, false, false),
        	tf.createFcItfType("c1", Compute3.class.getName(), true, false, false),
        	tf.createFcItfType("c2", Compute4.class.getName(), true, false, false)
        });
        
        Class klass = DispatcherClassGenerator.instance().generateClass(type);
        
		for(Class c: klass.getInterfaces()){
			System.out.println(c.getCanonicalName());
		}
		
	}
}
