package org.ercim.gridcomp.component.autonomic.controller;


import java.util.HashMap;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import org.ercim.gridcomp.component.autonomic.controller.collectiveitfs.CollectiveInterfaceDispatcherProxy;
import org.ercim.gridcomp.component.autonomic.controller.collectiveitfs.CollectiveInterfaceMessageEventListener;
import org.ercim.gridcomp.component.autonomic.controller.itf.AutonomicController;
import org.ercim.gridcomp.component.autonomic.manager.AutonomicComponentLCC;
import org.ercim.gridcomp.component.autonomic.manager.itf.AutonomicClientManager;
import org.ercim.gridcomp.component.autonomic.manager.itf.AutonomicServerManager;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;

import org.objectweb.proactive.ProActive;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.ContentDescription;
import org.objectweb.proactive.core.component.ControllerDescription;
import org.objectweb.proactive.core.component.Fractive;
import org.objectweb.proactive.core.component.ProActiveInterface;
import org.objectweb.proactive.core.component.controller.ComponentParametersController;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.representative.ProActiveComponentRepresentative;
import org.objectweb.proactive.core.component.type.ProActiveInterfaceType;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.descriptor.data.VirtualNode;
import org.objectweb.proactive.core.mop.ConstructionOfReifiedObjectFailedException;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;


public class AutonomicControllerForFarmImpl extends AbstractAutonomicController {

	private static final long serialVersionUID = -7541365150216315513L;
	private int VirtualNodeIndex = 1;
	private String DESCRIPTOR_URL = "deployment/deployment-descriptor.xml"; 
	private ProActiveDescriptor pad = null;
	
	protected int VirtualNodesNumber = 0;
	protected HashMap<Component, AutonomicController> autonomicControllersOfInternalComponents = new HashMap<Component, AutonomicController>();
	protected String className = AutonomicControllerForFarmImpl.class.getSimpleName();

	// Data structures to manage the names of the collective interfaces created automatically
	private HashMap<String, ProActiveInterface> itfNameToCollItf = new HashMap<String, ProActiveInterface>();
	private HashMap<String, CollectiveInterfaceDispatcherProxy> itfNameToCollectiveItfProxy = new HashMap<String, CollectiveInterfaceDispatcherProxy>();

	private CollectiveInterfaceMessageEventListener evtlist = new CollectiveInterfaceMessageEventListener();
	
	private ContentController ownerCC = null;
	private LifeCycleController ownerLCC = null;

	private boolean isControllerInitialized = false;
	private boolean isCollectiveInitialized = false;
	private int addedComponents = -1;
	
	public AutonomicControllerForFarmImpl(Component owner) {
		super(owner);
	}
	
	protected void out(String s){
		System.out.println(className+"> "+s); 
	}

	
	protected void initializeCollectiveInterfaces() throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException{
		
		isCollectiveInitialized = true;
		Interface[] ownerItfs = (Interface[]) owner.getRepresentativeOnThis().getFcInterfaces();
		BindingController ownerBC = (BindingController) owner.getFcInterface(Constants.BINDING_CONTROLLER);

		for (Interface itf : ownerItfs) {

			if (!((InterfaceType) itf.getFcItfType()).isFcClientItf()
					&& !itf.getFcItfName().endsWith("-controller")) {

				ProActiveInterface cCompCollItf = null;
				CollectiveInterfaceDispatcherProxy collectiveProxy = null;

				cCompCollItf = Fractive.createCollectiveClientInterface(itf
						.getFcItfName(), ((InterfaceType) itf.getFcItfType())
						.getFcItfSignature(), owner.getRepresentativeOnThis());

				itfNameToCollItf.put(itf.getFcItfName(), cCompCollItf);

				collectiveProxy = null;
				try {
					collectiveProxy = new CollectiveInterfaceDispatcherProxy( ( (InterfaceType) itf.getFcItfType()).getFcItfSignature() );
					collectiveProxy.setInterfaceType( (ProActiveInterfaceType) itf.getFcItfType() );

				} catch (ConstructionOfReifiedObjectFailedException e) {
					e.printStackTrace();
				}

				itfNameToCollectiveItfProxy.put(itf.getFcItfName(),
						collectiveProxy);

				
				collectiveProxy.add(filterComponents(ownerCC.getFcSubComponents()).getFcInterface(itf.getFcItfName()));
				cCompCollItf.setProxy(collectiveProxy);
				ownerBC.unbindFc(itf.getFcItfName());
				ownerBC.bindFc(itf.getFcItfName(), cCompCollItf);
				addedComponents = 0;
			}
			
		}
		
		evtlist.addIDtoObserve( ((ProActiveComponent)filterComponents(ownerCC.getFcSubComponents())).getID() );//ProActive.getBodyOnThis().getID());
		
	}
	
	
	// Controller initializer
	protected void initializeController(){
		
		try {
			ownerLCC = (LifeCycleController) owner.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
			ownerCC  = (ContentController)   owner.getFcInterface(Constants.CONTENT_CONTROLLER);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		isControllerInitialized = true;
		
		Component[] cmps = ownerCC.getFcSubComponents();
		for(Component c : cmps){
			
			if(c instanceof ProActiveComponent){
				ProActiveComponentRepresentative cRep;
				cRep = (ProActiveComponentRepresentative)
					   ((ProActiveComponent)c).
					   getRepresentativeOnThis();
				
				try {
					AutonomicController ac;
					ac = (AutonomicController) cRep.getFcInterface(AutonomicController.CONTROLLER_NAME);
					autonomicControllersOfInternalComponents.put(c, ac);
				} catch (NoSuchInterfaceException e) {
					
				}
			}
			
		}
		
	}
	

	// NF operation executor
	public GenericTypeWrapper execOperation(String op, Object... params) {

		if(!isControllerInitialized) initializeController();
		
		try {

			// check the operation requested
			if(op.equals("increaseParallelDegree")){
				
				// Suspends component lifecycle
				((AutonomicComponentLCC) ownerLCC).autonomicStopFc();
			
				// If the controller is not initialized, it initialize it
				if(!isCollectiveInitialized) initializeCollectiveInterfaces();
				
				// Retrieves one of internal components
				Component internal = filterComponents(ownerCC.getFcSubComponents());

				// Generates a clone of internal components
				Component newC = newComponent(internal);
				((LifeCycleController)newC.getFcInterface(Constants.LIFECYCLE_CONTROLLER)).stopFc();

				// Add the clones to the enclosing component
				ownerCC.addFcSubComponent(newC);
				
				// If the clone has an autonomiccontroller, it is retrieved and stored
				try {
					AutonomicController ac;
					ac = (AutonomicController) newC.getFcInterface(AutonomicController.CONTROLLER_NAME);
					autonomicControllersOfInternalComponents.put(newC, ac);
				} catch (NoSuchInterfaceException e) {
					
				}

				// If internal component is linked to some server interfaces, it binds the clone with them
				try{
					BindingController internalCompBC = (BindingController) internal.getFcInterface(Constants.BINDING_CONTROLLER);
					BindingController newCompBC 	 = (BindingController) newC.getFcInterface(Constants.BINDING_CONTROLLER);
					
					String[] internalCompItfs = internalCompBC.listFc();
					for(String internalCompItf : internalCompItfs){
						Interface itf = (Interface) internalCompBC.lookupFc(internalCompItf);
						newCompBC.bindFc(internalCompItf, itf);
					}
					
				} catch (NoSuchInterfaceException e) {
					out("Replicated component have not any client interface");
				}

				// Retrieves the binding controller and the other itfs by the owner
				//BindingController ownerBC = (BindingController) owner.getFcInterface(Constants.BINDING_CONTROLLER);
				Interface[] ownerItfs 	  = (Interface[]) 		owner.getRepresentativeOnThis().getFcInterfaces();

				// Binds each server interface of clone (except the controller ones) with a collective interface, creating it
				// if missing
				for(Interface itf : ownerItfs){

					if(!((InterfaceType)itf.getFcItfType()).isFcClientItf() && !itf.getFcItfName().endsWith("-controller")){

						//ProActiveInterface cCompCollItf = null;
						CollectiveInterfaceDispatcherProxy collectiveProxy = null;

						// If the current number of internal component is -1, it means that there aren't any collective itfs and 
						// it must create such ones
						/*if(addedComponents == -1){
							cCompCollItf = Fractive.createCollectiveClientInterface(itf.getFcItfName(),
									((InterfaceType)itf.getFcItfType()).getFcItfSignature(), 
									owner.getRepresentativeOnThis());

							itfNameToCollItf.put(itf.getFcItfName(), cCompCollItf);

							collectiveProxy = null;
							try {
								collectiveProxy = new CollectiveInterfaceDispatcherProxy(((InterfaceType)itf.getFcItfType()).getFcItfSignature());
								collectiveProxy.setInterfaceType((ProActiveInterfaceType)itf.getFcItfType());

							} catch (ConstructionOfReifiedObjectFailedException e) {
								e.printStackTrace();
							}

							itfNameToCollectiveItfProxy.put(itf.getFcItfName(), collectiveProxy);

						}else{*/
							//cCompCollItf = itfNameToCollItf.get(itf.getFcItfName());
							collectiveProxy = itfNameToCollectiveItfProxy.get(itf.getFcItfName());
						/*}*/

						// it bind the collective interface itf with the clone and with the enclosing component
						collectiveProxy.add(newC.getFcInterface(itf.getFcItfName()));
						/*if(addedComponents == -1){
							collectiveProxy.add(internal.getFcInterface(itf.getFcItfName()));
							cCompCollItf.setProxy(collectiveProxy);
							ownerBC.unbindFc(itf.getFcItfName());
							ownerBC.bindFc(itf.getFcItfName(), cCompCollItf);
							addedComponents = 1;
						}else{*/
							
						/*}*/

					}
				}

				addedComponents++;
				
				// Restarts the lifecycle of enclosing component
				((AutonomicComponentLCC) ownerLCC).autonomicStartFc();
				
				Vector<Object> res = new Vector<Object>();
				res.add(new Boolean(true));
				res.add(new Integer(addedComponents+1));
				return new GenericTypeWrapper<Vector<Object>>(res);
				
			} else if(op.equals("decreaseParallelDegree")){ 
				
				if(addedComponents > 0){

					// Suspends component lifecycle
					((AutonomicComponentLCC) ownerLCC).autonomicStopFc();
					
					// If the controller is not initialized, it initialize it
					if(!isCollectiveInitialized) initializeCollectiveInterfaces();

					// Retrieves the internal components
					Component[] internalComponents = ownerCC.getFcSubComponents();

					// Retrieves the last added internal component
					Component internal = internalComponents[internalComponents.length-1];

					// Retrieves the interfaces of composite component
					Interface[] ownerItfs = (Interface[]) owner.getRepresentativeOnThis().getFcInterfaces();

					// unbind the component to be removed from all the collective interfaces  
					for(Interface itf : ownerItfs){

						if(!((InterfaceType)itf.getFcItfType()).isFcClientItf() && !itf.getFcItfName().endsWith("-controller")){
							CollectiveInterfaceDispatcherProxy collectiveProxy = null;
							collectiveProxy = itfNameToCollectiveItfProxy.get(itf.getFcItfName());
							collectiveProxy.remove(internal.getFcInterface(itf.getFcItfName()));
						}
					}
					
					// remove the component to be removed from the component which contains it
					ownerCC.removeFcSubComponent(internal);
					
					// remove the ID of the component to be removed from the list of component to be observed
					evtlist.removeIDtoObserve(((ProActiveComponent) internal).getID());
					
					// If the clone has an autonomiccontroller, it is retrieved and removed from the store
					try {
						internal.getFcInterface(AutonomicController.CONTROLLER_NAME);
						autonomicControllersOfInternalComponents.remove(internal);
					} catch (NoSuchInterfaceException e) {
						
					}

					// Restarts the lifecycle of the enclosing component
					((AutonomicComponentLCC) ownerLCC).autonomicStartFc();
					
					addedComponents--;
					Vector<Object> res = new Vector<Object>();
					res.add(new Boolean(true));
					res.add(new Integer(addedComponents+1));
					return new GenericTypeWrapper<Vector<Object>>(res);
					
				}else{
					
					Vector<Object> res = new Vector<Object>();
					res.add(new Boolean(false));
					res.add(new Integer(addedComponents+1));
					return new GenericTypeWrapper<Vector<Object>>(res);
				}
				
				
			}else if(op.equals("JobEnqueuingTime")){
				
				// If the controller is not initialized, it initialize it
				if(!isCollectiveInitialized){
				
					// Suspends component lifecycle
					((AutonomicComponentLCC) ownerLCC).autonomicStopFc();
				
					initializeCollectiveInterfaces();
				
					// Restarts the lifecycle of the enclosing component
					((AutonomicComponentLCC) ownerLCC).autonomicStartFc();
				}
				
				if((params == null)||(params.length == 0)){
					String[] methodsName = evtlist.getObservedMethods();
					return new GenericTypeWrapper<String[]>(methodsName);
				}
				else if(params[0] instanceof String){
					String methodName = (String) params[0];
					long methodLatency = evtlist.getAverageLatencyInMillisOf(methodName);
					return new GenericTypeWrapper<Long>(methodLatency);
				}
				else if(params[0] instanceof Boolean){
					if((Boolean) params[0]){
						ProActive.getBodyOnThis().addMessageEventListener(evtlist);
					}else{
						ProActive.getBodyOnThis().removeMessageEventListener(evtlist);
					}
				}
				
			}
			
		} catch (NoSuchInterfaceException e) {
			System.out.println("No Such interface Exception");
			e.printStackTrace();
		} catch (IllegalContentException e) {
			System.out.println("Illegal Content Exception");
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			System.out.println("Illegal life cycle Exception");
			e.printStackTrace();
		} catch (IllegalBindingException e) {
			System.out.println("Illegal Binding Exception");
			e.printStackTrace();
		}	
		
		return new GenericTypeWrapper();
	}
	
	
	// Avoid the replication of Components which are Autonomic Manager
	private Component filterComponents(Component[] components) {
		
		int index = 0;
		boolean found = false;
		
		while((index < components.length) && (!found)){
			
			if(components[index] instanceof ProActiveComponent){
				ProActiveComponentRepresentative rep = 
					(ProActiveComponentRepresentative) 
					((ProActiveComponent) components[index]).getRepresentativeOnThis();
				
				if ( !(rep.getStubOnBaseObject() instanceof AutonomicServerManager)
					&& !(rep.getStubOnBaseObject() instanceof AutonomicClientManager)) {
					
					return components[index];
					
				}else{
					index++;
					continue;
				}
				
			}else{
				return components[index];
			}
		}
		
		return null;
	}
	
	
	private Component newComponent(Component toParallelize) {

		// Retreiving Type & Component Factories
		Component boot = null;
		try {
			boot = org.objectweb.fractal.api.Fractal.getBootstrapComponent();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}

		GenericFactory cf = null;
		try {
			cf = Fractal.getGenericFactory(boot);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Cloning controllers
		ControllerDescription controlDesc = null;
		try {
			controlDesc = ((ComponentParametersController) toParallelize.getFcInterface(Constants.COMPONENT_PARAMETERS_CONTROLLER)).
			getComponentParameters().
			getControllerDescription();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}

		// Creating a new Name
		NameController toParallelizeNC = null;
		try {
			toParallelizeNC = (NameController) toParallelize.getFcInterface(Constants.NAME_CONTROLLER);
		} catch (NoSuchInterfaceException e2) {
			e2.printStackTrace();
		}
		String name  = toParallelizeNC.getFcName()+"-autonomic-duplicate-"+(VirtualNodeIndex);
		controlDesc.setName(name);

		// Cloning content
		String clazz = ((ProActiveComponentRepresentative)toParallelize).getStubOnBaseObject().getClass().getSuperclass().getName();
		ContentDescription contentDesc = new ContentDescription(clazz);

		Component toParallelizeClone = null;
		try {
			toParallelizeClone = cf.newFcInstance( toParallelize.getFcType(), controlDesc, contentDesc );
			
			if(pad == null){
				pad = ProActive.getProactiveDescriptor(DESCRIPTOR_URL);
				VirtualNode[] vnodes = pad.getVirtualNodes();
				for(VirtualNode vn : vnodes){
					if(vn.getName().startsWith(toParallelizeNC.getFcName()+"-vn-")){
						VirtualNodesNumber++;
					}
				}
				if(VirtualNodesNumber == 0) throw new ProActiveException("No such virtual nodes whose name start with: "+toParallelizeNC.getFcName()+"-vn-");
			}
			
			VirtualNode vn = pad.getVirtualNode(toParallelizeNC+"-vn-"+ ((VirtualNodeIndex++) % VirtualNodesNumber) );
			toParallelizeClone = ( new Fractive() ).newFcInstance(toParallelize.getFcType(), controlDesc, contentDesc, vn);
			
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (ProActiveException e) {
			e.printStackTrace();
			System.exit(1);
		}

		LifeCycleController tPcLCC;
		try {
			tPcLCC = (LifeCycleController) toParallelizeClone.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
			tPcLCC.stopFc();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			e.printStackTrace();
		}

		evtlist.addIDtoObserve(((ProActiveComponent)toParallelizeClone).getID());

		return toParallelizeClone;
	}


	public String[] listAutonomicOperations() {
		
		if(!isControllerInitialized) initializeController();
		
		Set<String> operationsFromInternalComponents = new TreeSet<String>();
		
		try {
		
			// If the controller is not initialized, it initialize it
			if(!isCollectiveInitialized){
				// Suspends component lifecycle
				((AutonomicComponentLCC) ownerLCC).autonomicStopFc();
			
				initializeCollectiveInterfaces();
			
				// Restarts the lifecycle of the enclosing component
				((AutonomicComponentLCC) ownerLCC).autonomicStartFc();
			}
		
			operationsFromInternalComponents.add("increaseParallelDegree");
			operationsFromInternalComponents.add("decreaseParallelDegree");
			operationsFromInternalComponents.add("JobEnqueuingTime");
		
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		} catch (IllegalBindingException e) {
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			e.printStackTrace();
		}
	
		return operationsFromInternalComponents.toArray(new String[operationsFromInternalComponents.size()]);
	}

}
