package org.ercim.gridcomp.component.autonomic.controller.collectiveitfs;

import java.lang.reflect.InvocationTargetException;

import org.objectweb.proactive.Body;
import org.objectweb.proactive.ProActive;
import org.objectweb.proactive.core.body.LocalBodyStore;
import org.objectweb.proactive.core.component.group.ProxyForComponentInterfaceGroup;

import org.objectweb.proactive.core.exceptions.manager.NFEManager;
import org.objectweb.proactive.core.exceptions.proxy.FailedGroupRendezVousException;

import org.objectweb.proactive.core.group.ExceptionListException;
import org.objectweb.proactive.core.group.ProcessForOneWayCall;

import org.objectweb.proactive.core.mop.ConstructionOfReifiedObjectFailedException;
import org.objectweb.proactive.core.mop.MethodCall;

import org.objectweb.proactive.core.util.profiling.Profiling;


public class CollectiveInterfaceDispatcherProxy extends ProxyForComponentInterfaceGroup {

	private static final long serialVersionUID = 7616783957337770905L;
	private int currentWorker = 0; 
	
	public CollectiveInterfaceDispatcherProxy(String nameOfClass) throws ConstructionOfReifiedObjectFailedException {
		super(nameOfClass);
		
	}
	
	boolean initialized = false;

	@Override
	protected Object asynchronousCallOnGroup(MethodCall mc) throws InvocationTargetException {
			
		Body body = ProActive.getBodyOnThis();
	
		// Creating Threads
		if (uniqueSerialization) {
			mc.transformEffectiveArgumentsIntoByteArray();
		}
		
		int nextWorker = nextWorker();
		ProcessForSingleAsynchronousCall pfac = new ProcessForSingleAsynchronousCall(this, this.memberList, null, nextWorker, mc, body);
		this.threadpool.addAJob(pfac);
		
		//return result;
		Object ret = pfac.getResult();

		this.threadpool.checkNumberOfThreads(memberList.size());
		
		if(ret instanceof Throwable) {
			((Throwable) ret).printStackTrace();
			throw new InvocationTargetException((Throwable) ret);
		}
	
		return ret;
	}

	@Override
	protected void oneWayCallOnGroup(MethodCall mc, ExceptionListException exceptionList) throws InvocationTargetException {

		Body body = ProActive.getBodyOnThis();
		if (Profiling.GROUP) {
			timer.setTimer("oneWayCallOnGroup." + mc.getName());
			timer.start();
		}
		
		
		// Creating Threads
		if (uniqueSerialization) {
			mc.transformEffectiveArgumentsIntoByteArray();
		}

		this.threadpool.addAJob(new ProcessForOneWayCall(this,
				this.memberList, nextWorker(), mc, body, exceptionList));
		
		LocalBodyStore.getInstance().setCurrentThreadBody(body);

		if (Profiling.GROUP) {
			timer.stop();
		}
	}

	
	private int nextWorker() {
		return ((currentWorker++) % memberList.size());
	}

	
	@Override
	public synchronized Object reify(MethodCall mc)
	throws InvocationTargetException  {
		ExceptionListException exceptionList = null;

		/* if the method called is toString, apply it to the proxy, not to the members */
		if ("toString".equals(mc.getName())) {
			return this.toString();
		}

		/* if the method called is hashCode, apply it to the proxy, not to the members */
		if ("hashCode".equals(mc.getName())) {
			return new Integer(this.hashCode());
		}

		if ("equals".equals(mc.getName()) && (mc.getNumberOfParameter() == 1)) {
			return new Boolean(this.equals(mc.getParameter(0)));
		}

		/* result will be a stub on a proxy for group representing the group of results */
		Object result = null;

		/* if OneWay : do not construct result */
		if (mc.isOneWayCall()) {
			exceptionList = new ExceptionListException();
			this.oneWayCallOnGroup(mc, exceptionList);
		}
		/* Special case : the method returns void but is Synchronous because it throws Exception */
		else if (mc.getReifiedMethod().getReturnType() == Void.TYPE) {
			exceptionList = new ExceptionListException();
			this.oneWayCallOnGroup(mc, exceptionList);
		}
		/* if the call is asynchronous the group of result will be a group a future */
		else { // with group in general case : SYNC == ASYNC !!!!
			result = this.asynchronousCallOnGroup(mc);
		}

		/* A barrier of synchronisation to be sur that all calls are done before continuing the execution */
		this.threadpool.complete();

		/* Throws the exceptionList if one or more exceptions occur in the oneWayCall */
		if ((exceptionList != null) && (exceptionList.size() != 0)) {
			FailedGroupRendezVousException fgrve = new FailedGroupRendezVousException(
					"RendezVous failed in group method: " + mc.getName(),
					exceptionList, this);
			NFEManager.fireNFE(fgrve, this);
		}

		return result;
	}

	@Override
	protected void redistributeParameters(MethodCall mc) {

		super.redistributeParameters(mc);
	}
}
