package org.ercim.gridcomp.component.autonomic.controller.collectiveitfs;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import org.objectweb.proactive.core.UniqueID;
import org.objectweb.proactive.core.event.MessageEvent;
import org.objectweb.proactive.core.event.MessageEventListener;

public class CollectiveInterfaceMessageEventListener implements MessageEventListener,Serializable {

	private static final long serialVersionUID = 6619123495040842543L;
	private final int INVOCATION_STORED = 5;
	
	private HashMap<String, HashMap<Long, Long> > findTimeMap = new HashMap<String, HashMap<Long,Long>>();
	private HashMap<String, TimeElapsedPerInvocation> averageMethodsLatencyInMillis = new HashMap<String, TimeElapsedPerInvocation>();
	
	private HashMap<String, Object> controlConstants = new HashMap<String, Object>();
	
	private Set<UniqueID> IDSet = new HashSet<UniqueID>();

	public void addIDtoObserve(UniqueID id){
		IDSet.add(id);
	}
	
	public void removeIDtoObserve(UniqueID id){
		IDSet.remove(id);
	}
	
	public CollectiveInterfaceMessageEventListener(){
		
		// LCC
		controlConstants.put("startFc", new Object());
		controlConstants.put("stopFc", new Object());
		controlConstants.put("getFcState", new Object());

		// ABC
		controlConstants.put("listAutonomicOperations", new Object());
		controlConstants.put("execOperation", new Object());

		// CPC
		controlConstants.put("getComponentParameters", new Object());
		controlConstants.put("setComponentParameters", new Object());
		
		
		controlConstants.put("getFcSuperComponents", new Object());
		controlConstants.put("getFcSubComponents", new Object());
		
		// NC
		controlConstants.put("getFcName", new Object());
		controlConstants.put("setFcName", new Object());
		controlConstants.put("setControllerItfType", new Object());
		
		// CC
		controlConstants.put("addFcSubComponent", new Object());
		controlConstants.put("removeFcSubComponent", new Object());
		controlConstants.put("getFcInternalInterface", new Object());
		controlConstants.put("getFcInternalInterfaces", new Object());
		
		// BC
		controlConstants.put("bindFc", new Object());
		controlConstants.put("unbindFc", new Object());
		controlConstants.put("listFc", new Object());
		controlConstants.put("lookupFc", new Object());

		controlConstants.put("hashCode", new Object());
		
		// PAC
		controlConstants.put("getFcItfName", new Object());
		controlConstants.put("getFcItfOwner", new Object());
		controlConstants.put("getFcItfType", new Object());
		controlConstants.put("isFcInternalItf", new Object());
		controlConstants.put("migrateDependentActiveObjectsTo", new Object());
		
		// SC
		controlConstants.put("addParent", new Object());
		controlConstants.put("removeParent", new Object());
		
	}

	private boolean requiresAnalysis(String methodName, UniqueID uniqueID) {
		
		boolean isControlMethod = controlConstants.containsKey(methodName);
		boolean isIDtoObserve = IDSet.contains(uniqueID);
		
		return ((!isControlMethod)&&(isIDtoObserve));
		
	}
	
	public synchronized void replyReceived(MessageEvent event) {

		if(requiresAnalysis(event.getMethodName(), event.getSourceBodyID())){

			long time = 0;
			try{
				HashMap<Long, Long> timeMap = findTimeMap.get(event.getMethodName());
				time = timeMap.get(event.getSequenceNumber());
				time = event.getTimeStamp() - time;

				averageMethodsLatencyInMillis.get(event.getMethodName()).insert(time);
				
			}catch(Exception e){}
		}
	}

	public void requestSent(MessageEvent event) {

		if(requiresAnalysis(event.getMethodName(), event.getDestinationBodyID())){
			
			HashMap<Long, Long> timeMap = null; 
			
			if(!findTimeMap.containsKey(event.getMethodName())){
				timeMap = new HashMap<Long, Long>();
				findTimeMap.put(event.getMethodName(), timeMap);
				averageMethodsLatencyInMillis.put(event.getMethodName(), new TimeElapsedPerInvocation(INVOCATION_STORED));
			}else{
				timeMap = findTimeMap.get(event.getMethodName());
			}
			
			long time = event.getTimeStamp();
			timeMap.put(event.getSequenceNumber(), time);
		}
	}

	public String[] getObservedMethods() {
		return findTimeMap.keySet().toArray(new String[findTimeMap.keySet().size()]);
	}

	public long getAverageLatencyInMillisOf(String methodName){
		return averageMethodsLatencyInMillis.get(methodName).average();
	}

	public void replySent(MessageEvent event) { }

	public void requestReceived(MessageEvent event) { }
	
	public void servingStarted(MessageEvent event) { }

	public void voidRequestServed(MessageEvent event) { }
}


class TimeElapsedPerInvocation extends LinkedList<Long>{

	private static final long serialVersionUID = -5682999676512798244L;
	private int storedInvocation = 0;
	private double averageValue = 0;
	
	public TimeElapsedPerInvocation(int invocationToBeStored){
		storedInvocation = invocationToBeStored;
	}
	
	
	void insert(Long elem){
		
		if(size() >= storedInvocation){
			
			Long removed = remove();
			add(elem);
			
			double removedContribute = ((double) removed) / storedInvocation;
			double addedContribute = ((double) elem) / storedInvocation;
			averageValue += addedContribute - removedContribute;
		
		}else{
			
			add(elem);
			double addedContribute = ((double) elem) / size();
			
			averageValue = ( (averageValue * (size() - 1)) / size() ) + addedContribute;
			
		}
	}

	Long average(){
		return Math.round(averageValue);
	}
}
