package org.ercim.gridcomp.component.autonomic.manager;

import org.ercim.gridcomp.component.autonomic.controller.itf.AutonomicController;
import org.ercim.gridcomp.component.autonomic.manager.itf.AutonomicClientManager;
import org.ercim.gridcomp.component.autonomic.manager.itf.AutonomicServerManager;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

import org.objectweb.proactive.Body;
import org.objectweb.proactive.RunActive;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;


public abstract class AbstractAutonomicManager implements RunActive,
		AutonomicServerManager, AutonomicClientManager, BindingController {

	protected long sleepTime = 6000;
	protected boolean isManagerInitialized = false;
	protected AutonomicController managedAutonomicController = null;
	
	protected String className = AbstractAutonomicManager.class.getSimpleName();
	protected void out(String s){ System.out.println(className+"> "+s); }
	protected void err(String s){ System.err.println(className+"> "+s); }
	
	private String currentQoSContract;
	
	
	// Autonomic Manager Main Loop
	public void runActivity(Body arg0) {
		
		// Initializing Autonomic Manager
		if(!isManagerInitialized) 
			initAutonomicManager();

		// Sleep "sleepTime" seconds 
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		// Enforcing Performance Contract
		try {
			enforcePerformanceContract(currentQoSContract);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	// Autonomic Manager Performance Contract Enforcement
	protected abstract void enforcePerformanceContract(String qosContract);
	
	
	// Autonomic Manager Initialization
	protected void initAutonomicManager(){
		isManagerInitialized = true;
	}
	
	
  /****  AutonomicServerManager SERVER-INTERFACE (External interface)  ***/
	
	public GenericTypeWrapper commitContract(String qosContract) {
		currentQoSContract = qosContract;
		return null;
	}
	
  /***********************************************************************/

	
	
	
  /****  AutonomicClientManager SERVER-INTERFACE  (Internal interface) ***/
	
	public GenericTypeWrapper raiseViolation(GenericTypeWrapper violationId) {
		return null;
	}
	
  /***********************************************************************/
	
	
	
  /****  BINDING CONTROLLER  ****/
	
 	public void bindFc(String itfName, Object itf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		
		if(itfName == null) throw new NoSuchInterfaceException("Interface name cannot be 'null'");
		if(itf == null) throw new IllegalBindingException("Interface binded cannot be 'null'");
		
		if(itfName.equals("client-autonomic-controller")){
			managedAutonomicController = (AutonomicController) itf;
		}
	}
	
	public String[] listFc() {
		return new String[]{"client-autonomic-controller"};
	}

	public Object lookupFc(String itfName) throws NoSuchInterfaceException {
		
		if(itfName == null) throw new NoSuchInterfaceException("Interface name cannot be 'null'");
		
		if(itfName.equals("client-autonomic-controller")){
			Object result = managedAutonomicController;
			if(result != null) 
				return result;
		}
		
		throw new NoSuchInterfaceException(itfName);
	}

	public void unbindFc(String itfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
		
		if(itfName == null) throw new NoSuchInterfaceException("Interface name cannot be 'null'");
		
		if(itfName.equals("client-autonomic-controller")){
			managedAutonomicController = null;
		}else
			throw new NoSuchInterfaceException(itfName);
	}
	
  /******************************/

}
