package passivefarm;

import java.util.HashMap;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.util.Fractal;

import org.objectweb.proactive.ProActive;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.Fractive;
import org.objectweb.proactive.core.component.controller.ProActiveContentController;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.representative.ProActiveComponentRepresentative;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;

import org.ercim.gridcomp.component.autonomic.controller.itf.AutonomicController;
import passivefarm.itfs.Tester;

public class Main {

	private static final long serialVersionUID = 1L;

    public void action() throws Exception {

        Factory f = org.objectweb.proactive.core.component.adl.FactoryFactory.getFactory();
        System.setProperty("proactive.pad", "passivefarm/deployment/deployment-descriptor.xml");
        
        ProActiveDescriptor deploymentDescriptor = ProActive.getProactiveDescriptor("passivefarm/deployment/deployment-descriptor.xml");
        deploymentDescriptor.activateMappings(); 
        
        HashMap<String,Object> context = new HashMap<String,Object>(1);
        context.put("deployment-descriptor", deploymentDescriptor);
        
        Component testcase = (Component) f.newComponent("passivefarm.adl.application", context);
        Fractal.getLifeCycleController(testcase).startFc();
        
        ProActiveContentController testcaseCC = Fractive.getProActiveContentController(testcase);
        Component[] testcaseInternalComponents = testcaseCC.getFcSubComponents();
        
        ProActiveComponent Cmp = null;
        AutonomicController ac = null;
        for(Component cmp : testcaseInternalComponents){
        	
        	NameController nc = (NameController) cmp.getFcInterface(Constants.NAME_CONTROLLER);
        	
        	if(nc.getFcName().equals("farm")){
        		
        		ProActiveComponentRepresentative pcr = 
        			(ProActiveComponentRepresentative)((ProActiveComponent) cmp).getRepresentativeOnThis();
        			
        		ac = (AutonomicController) pcr.getFcInterface(AutonomicController.CONTROLLER_NAME);     
        	    Cmp = (ProActiveComponent) cmp;	
        	}
     
        }
        
        if(Cmp != null){
        	
        		System.out.println("\t\t ----- GridCOMP Application ----- ");
        		System.out.println("\t\t      Passive Autonomic Farm      ");
        		System.out.println("\t\t----------------------------------");
        		Thread.sleep(1500);
        		System.out.println();
    			System.out.println(">>> Let us start with a single worker... <<<");
    			Thread.sleep(4000);
    			System.out.println();
    			
    			
    			long time = 0;
    			int job = 0;
    			char c = '\0';	
    			for(int j=0;j<200;j++, job++){
    				
       				long tmp_time = System.currentTimeMillis();
    				((Tester)testcase.getFcInterface("runTestItf")).testItf(1024, 8);
    				time += (System.currentTimeMillis() - tmp_time);
    				
    				if(System.in.available() > 0){
    					c = (char) System.in.read();
    				
    					if(c == '+'){
    						System.out.println("Increasing Parallel Degree");
    						ac.execOperation("increaseParallelDegree", new Object[0]);
    					}else if(c == '-'){
    						System.out.println("Decreasing Parallel Degree");
    						ac.execOperation("decreaseParallelDegree", new Object[0]);
    					}
    				}
    				if(job == 10){
    					System.out.println("Last 10 jobs required "+time+" milliseconds.");
    					time = 0;
    					job = 0;
    				}
    			}
        	
        }
        
        System.out.println("Process Terminated");
        System.exit(0);
	}

   
    
    public static void main(String[] args) {
      
    	
        Main test = new Main();
        try {
        	test.action();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
